/* 
 * Copyright (c) 2011, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/*
 *  ======== itest.h ========
 *  ITEST Interface Header
 *
 */
#ifndef ITEST_
#define ITEST_

#include <ti/xdais/ialg.h>
#include <ti/xdais/idma2.h>

#define TEST_SOK      0
#define TEST_EFAIL    -1
#define TEST_EMEMORY  -2


/*
 *  ======== ITEST_HANDLE ========
 *  This handle is used to reference all TEST instance objects
 */
typedef struct ITEST_Obj *ITEST_Handle;

/*
 *  ======== ITEST_Obj ========
 *  This structure must be the first field of all TEST instance objects
 */
typedef struct ITEST_Obj {
    struct ITEST_Fxns *fxns;
} ITEST_Obj;

/*
 *  ======== ITEST_Cmd ========
 *  The Cmd enumeration defines the control commands for the TEST
 *  control method.
 */
typedef enum ITEST_Cmd {
    ITEST_GETTEST,      /* Get command */
    ITEST_SETTEST       /* Set command */
} ITEST_Cmd;

/*
 *  ======== ITEST_Params ========
 *  This structure defines the creation parameters for all TEST objects
 */
typedef struct ITEST_Params {
    Int size;           /* must be first field of all params structures */
    Int scratchGroup;   /* Scratch group the algorithm belongs to */
    Uns scratchSize;    /* size of scratch memory buffer to request */
    Uns value;          /* value to put in scratch memory */
    Int id;             /* ID of task the created the alg */
    IALG_MemRec *memTab; /* Array of memory requests specified by framework */
    Int numRecs;        /* Number of records in memRec */
    IALG_Fxns *fxns;    /* Pointer to IALG_Fxns (so we can check in algAlloc
                         * if algActivate() and algDeactivate() are non-NULL.
                         * (Can't use scratch if one of these is NULL).
                         */
} ITEST_Params;

/*
 *  ======== ITEST_PARAMS ========
 *  Default parameter values for TEST instance objects
 */
extern ITEST_Params ITEST_PARAMS;

/*
 *  ======== ITEST_Fxns ========
 *  This structure defines all of the operations on TEST objects
 */
typedef struct ITEST_Fxns {
    IALG_Fxns   ialg;    /* ITEST extends IALG */
    Int         (* useScratch)(IALG_Handle alg);
    Int         (* getBufs)(IALG_Handle alg, IALG_MemRec *memTab, Int nRecs);
} ITEST_Fxns;

#endif  /* ITEST_Fxns */
/*
 *  @(#) ti.sdo.fc.dskt2.examples; 1, 0, 0,118; 3-1-2011 18:57:24; /db/atree/library/trees/fc/fc-n29x/src/ xlibrary

 */

