/* 
 * Copyright (c) 2011, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/*
 *  ======== test0.c ========
 *
 *  Create and run algorithm that use DSKT2 to allocate persistent and
 *  scratch memory.
 */


#include <xdc/std.h>
#include <xdc/runtime/Diags.h>
#include <xdc/runtime/Memory.h>
#include <xdc/runtime/System.h>

#include <ti/sdo/fc/dskt2/dskt2.h>

#include <test_ti.h>

#define MOD_NAME DSKT2_MODNAME

extern Void smain(Int argc, Char * argv[]);

/*
 *  ======== smain ========
 */
/* ARGSUSED */
Void smain(Int argc, Char * argv[])
{
    ITEST_Handle  alg;
    ITEST_Params  params = ITEST_PARAMS;
    IALG_Fxns          *fxns;
    Int                 scratchId = 1;


    System_printf("test0> Started.\n");

    fxns = (IALG_Fxns *)&TEST_TI_ITEST;
    params.value = 1;           /* Value to put in scratch memory */
    params.fxns = fxns;

    /* Create, activate, and run alg */
    alg = (ITEST_Handle)DSKT2_createAlg(scratchId, fxns, NULL,
            (IALG_Params *)&params);

    /* Set trace for DSKT2 module */
    Diags_setMask(MOD_NAME"+4567");
    Diags_setMask(MOD_NAME"-EX123");

    printf("ALG is 0x%x\n", alg);
}
/*
 *  @(#) ti.sdo.fc.dskt2.examples; 1, 0, 0,118; 3-1-2011 18:57:25; /db/atree/library/trees/fc/fc-n29x/src/ xlibrary

 */

