/* 
 * Copyright (c) 2011, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */
/*
 *  ======== test_ti.c ========
 *  Implementation of the TEST_TI.h interface; TI's implementation 
 *  of the ITEST interface. 
 */

#ifdef __TI_COMPILER_VERSION__
#pragma CODE_SECTION(TEST_TI_alloc, ".text:algAlloc")
#pragma CODE_SECTION(TEST_TI_free, ".text:algFree")
#pragma CODE_SECTION(TEST_TI_initObj, ".text:algInit")
#pragma CODE_SECTION(TEST_TI_init, ".text:init")
#pragma CODE_SECTION(TEST_TI_exit, ".text:exit")
#pragma CODE_SECTION(TEST_TI_useScratch, ".text:test")
#endif

#include <xdc/std.h>
#include <xdc/runtime/System.h>

#include "itest.h"
#include "test_ti_priv.h"
#include "test_ti.h"


static Int verifyScratch(TEST_TI_Handle test);


/*
 *  ======== TEST_TI_activate ========
 *  Fills the scratch buffers.
 */
Void TEST_TI_activate(IALG_Handle handle)
{
}

/*
 *  ======== TEST_TI_alloc ========
 *  Return a table of memory descriptors that describe the memory needed 
 *  to construct a TEST_TI_Obj structure.
 */
/* ARGSUSED */
Int TEST_TI_alloc(const IALG_Params *testParams,
        IALG_Fxns **parentFxns, IALG_MemRec memTab[])
{

    /* Request memory for TEST object */
    memTab[0].size = 0xd0; ///sizeof(TEST_TI_Obj);
    memTab[0].alignment = 0x8;
    memTab[0].space = IALG_EXTERNAL; //IALG_DARAM0;
    memTab[0].attrs = IALG_PERSIST;

    memTab[1].size = 0x10000; ///sizeof(TEST_TI_Obj);
    memTab[1].alignment = 0x8;
    memTab[1].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[1].attrs = IALG_SCRATCH;

    memTab[2].size = 0x10000; ///sizeof(TEST_TI_Obj);
    memTab[2].alignment = 0x8;
    memTab[2].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[2].attrs = IALG_SCRATCH;

    memTab[3].size = 0x8000; ///sizeof(TEST_TI_Obj);
    memTab[3].alignment = 0x8;
    memTab[3].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[3].attrs = IALG_SCRATCH;

    memTab[4].size = 0x82a0; ///sizeof(TEST_TI_Obj);
    memTab[4].alignment = 0x8;
    memTab[4].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[4].attrs = IALG_SCRATCH;

    memTab[5].size = 0x1000; ///sizeof(TEST_TI_Obj);
    memTab[5].alignment = 0x8;
    memTab[5].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[5].attrs = IALG_PERSIST;

    memTab[6].size = 0x1000; ///sizeof(TEST_TI_Obj);
    memTab[6].alignment = 0x8;
    memTab[6].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[6].attrs = IALG_PERSIST;

    memTab[7].size = 0x8000; ///sizeof(TEST_TI_Obj);
    memTab[7].alignment = 0x8;
    memTab[7].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[7].attrs = IALG_SCRATCH;

    memTab[8].size = 0x8000; ///sizeof(TEST_TI_Obj);
    memTab[8].alignment = 0x8;
    memTab[8].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[8].attrs = IALG_PERSIST;

    memTab[9].size = 0x2000; ///sizeof(TEST_TI_Obj);
    memTab[9].alignment = 0x8;
    memTab[9].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[9].attrs = IALG_PERSIST;

    memTab[10].size = 0x2000; ///sizeof(TEST_TI_Obj);
    memTab[10].alignment = 0x8;
    memTab[10].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[10].attrs = IALG_PERSIST;

    memTab[11].size = 0x4000; ///sizeof(TEST_TI_Obj);
    memTab[11].alignment = 0x8;
    memTab[11].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[11].attrs = IALG_SCRATCH;

    memTab[12].size = 0x4000; ///sizeof(TEST_TI_Obj);
    memTab[12].alignment = 0x8;
    memTab[12].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[12].attrs = IALG_SCRATCH;

    memTab[13].size = 0x1000; ///sizeof(TEST_TI_Obj);
    memTab[13].alignment = 0x8;
    memTab[13].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[13].attrs = IALG_SCRATCH;

    memTab[14].size = 0x800; ///sizeof(TEST_TI_Obj);
    memTab[14].alignment = 0x8;
    memTab[14].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[14].attrs = IALG_SCRATCH;

    memTab[15].size = 0x800; ///sizeof(TEST_TI_Obj);
    memTab[15].alignment = 0x8;
    memTab[15].space = IALG_DARAM0; //IALG_DARAM0;
    memTab[15].attrs = IALG_SCRATCH;

    return (NUMALLOC);
}

/*
 *  ======== TEST_TI_control ========
 *  Execute a control operation.
 */
/* ARGSUSED */
Int TEST_TI_control(IALG_Handle handle, IALG_Cmd cmd,
        IALG_Status * status)
{
    return (1);
}

/*
 *  ======== TEST_TI_deactivate ========
 *  Checks that the scratch buffers still hold the correct values.
 */
Void TEST_TI_deactivate(IALG_Handle handle)
{
}

/*
 *  ======== TEST_TI_free ========
 *  Return a table of memory pointers that should be freed.  Note
 *  that this should include *all* memory requested in the 
 *  TEST_TI_alloc operation above.
 */
Int TEST_TI_free(IALG_Handle handle, IALG_MemRec memTab[])
{
    TEST_TI_Obj *test = (Void *)handle;
    Int             i;
    Int             n;

   /* Get the memory attributes for the TEST object */
    n = TEST_TI_alloc(NULL, NULL, memTab);

    /* Set the memory bases */
    memTab[0].base = test;
    for (i = 1; i < n; i++) {
        memTab[i].base = test->scratchBufs[i - 1];
    }

    return (n);
}


/*
 *  ======== TEST_TI_initObj ========
 *  Initialize the memory allocated for our instance.
 */
/* ARGSUSED */
Int TEST_TI_initObj(IALG_Handle handle, const IALG_MemRec memTab[], 
        IALG_Handle p, const IALG_Params *testParams)
{
#if 0
    TEST_TI_Obj         *test = (Void *)handle;
    const ITEST_Params  *params = (Void *)testParams;
    Int                       i;
    Int                       numBufs;

    numBufs = (params->numRecs > 0) ? params->numRecs - 1 : NUMBUFS;
        
    for (i = 0; i < numBufs; i++) {
        test->scratchBufs[i] = memTab[i + 1].base;
        test->bufSizes[i] = memTab[i + 1].size;

        /* Save memTab */
        test->memTab[i + 1] = memTab[i + 1];
    }

    test->memTab[0] = memTab[0];

    test->value = params->value;
    test->bufSize = params->scratchSize;
    test->scratchGroup = params->scratchGroup;
    test->id = params->id;
    test->numRecs = numBufs + 1;
    test->numBufs = numBufs;

#endif
    return (IALG_EOK);
}

/*
 *  ======== TEST_TI_numAlloc ========
 *  Return the maximum number memory blocks needed.
 */
Int TEST_TI_numAlloc(Void)
{
    return (16);
}
/*
 *  @(#) ti.sdo.fc.dskt2.examples; 1, 0, 0,118; 3-1-2011 18:57:25; /db/atree/library/trees/fc/fc-n29x/src/ xlibrary

 */

