/* 
 * Copyright (c) 2011, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

print("Platforms.xs: loaded from "+xdc.csd()); 

/* Utility function to ensure a platform is only added to a targ array once */
function addUniquePlatform(targ, plat, test_dsp) {
    var addPlat = plat; 
    print("addUniquePlatform " + targ + " " +addPlat); 
    var found = false;

    if (test_dsp != undefined) {
        addPlat = plat + ":test_dsp"; 
    }

    for (var i = 0; i < targ.platforms.length; i++) {
        if (targ.platforms[i] == addPlat) {
            found = true;
            break;
        }
    }
    if (!found) {
        if (addPlat.match(/test_dsp/)) {
            Build.platformTable[addPlat] = {l1DMode:'8k'};
            targ.platforms[targ.platforms.length++] = addPlat;
        }
        else {
            targ.platforms[targ.platforms.length++] = addPlat;
        }
    }
}

/* GUNTODO Need some better way to add these, maybe build them everytime ?! */  
function addGPPOS(thePlatform) {
    print("addGPPOS " + thePlatform); 

    if (gppos.match(/LINUX_GCC/)) {
        if (thePlatform.match(/host\.platforms\.PC/)) {
            /* This is X86 Linux - not sure the special case plat is good... */
            Linux86 = xdc.useModule('gnu.targets.Linux86');
            addUniquePlatform(Linux86, thePlatform);
        } else {
            /* assume it's an ARM */
            GCArmv5T = xdc.useModule('gnu.targets.arm.GCArmv5T');
            addUniquePlatform(GCArmv5T, thePlatform);
        }
    }
/*
    if (gppos.match(/LINUX_UCLIBC/)) {
        UCArm9 = xdc.useModule('gnu.targets.UCArm9');
        addUniquePlatform(UCArm9, thePlatform);
    }
*/
}

/*
 * Match all words and trim off leading and trailing white space,
 * then split on white space (any amount) between words.
 */
var deviceArray = devices.match(/^\s*(.*\w+)\s*$/)[1].split(/\s+/);

/*
 * The allDevices[] array maps the DEVICES macro set in xdcpaths.mak with
 * XDC platforms.
 */
var allDevices = new Array();
allDevices['DM6446']   = {platform: 'ti.platforms.evmDM6446',   GPP:true,  DSP:'C64P'};
allDevices['C6455']    = {platform: 'ti.platforms.dsk6455',     GPP:false,  DSP:'C64P'};
allDevices['DM6467']   = {platform: 'ti.platforms.evmDM6467',   GPP:true,  DSP:'C64P'};
allDevices['DM6467T']  = {platform: 'ti.platforms.evmDM6467T',  GPP:true,  DSP:'C64P'};
allDevices['DM6437']   = {platform: 'ti.platforms.evmDM6437',   GPP:false, DSP:'C64P'};
allDevices['DM648']    = {platform: 'ti.platforms.evmDM648',    GPP:false, DSP:'C64P'};
allDevices['DM355']    = {platform: 'ti.platforms.evmDM355',    GPP:true             };
allDevices['DM357']    = {platform: 'ti.platforms.evmDM357',    GPP:true             };
allDevices['DM365']    = {platform: 'ti.platforms.evmDM365',    GPP:true             };
allDevices['OMAP3530'] = {platform: 'ti.platforms.evm3530',     GPP:true,  DSP:'C64P'};
allDevices['OMAP3430'] = {platform: 'ti.platforms.evm3430',     GPP:true,  DSP:'C64P'};
allDevices['OMAP2530'] = {platform: 'ti.platforms.evm2530',     GPP:true,  DSP:'C64P'};
allDevices['OMAP2430'] = {platform: 'ti.platforms.sdp2430',     GPP:true,  DSP:'C64P'};
allDevices['OMAPL137'] = {platform: 'ti.platforms.evmOMAPL137', GPP:true,  DSP:'C674'};
allDevices['OMAPL138'] = {platform: 'ti.platforms.evmOMAPL138', GPP:true,  DSP:'C674'};
allDevices['X86']      = {platform: 'host.platforms.PC',        GPP:true             };
allDevices['DA830']    = {platform: 'ti.platforms.evmDA830',    GPP:true,  DSP:'C674'}; 
allDevices['DM8168CATALOG']   = {platform: 'ti.platforms.evmDM8168',   GPP:true,  DSP:'C674', ARM:'v7M'}; 
allDevices['OMAP4430'] = {platform: 'ti.platforms.sdp4430',     GPP:true,  DSP:'C674', ARM:'v7M'};
allDevices['C6472']    = {platform: 'ti.platforms.evm6472',     GPP:false,  DSP:'C64P'};
allDevices['C6474']    = {platform: 'ti.platforms.evm6474',     GPP:false,  DSP:'C64P'};
allDevices['DM8168']   = {platform: 'ti.platforms.evmDM8168',   GPP:true,  DSP:'C674'}; 
allDevices['C6A8168']  = {platform: 'ti.platforms.evmC6A8168',  GPP:true,  DSP:'C674'}; 
allDevices['DM8148']   = {platform: 'ti.platforms.evmDM8148',   GPP:true,  DSP:'C674'}; 

for (var i in deviceArray) {
    if (allDevices[deviceArray[i]] != undefined) {
        if (allDevices[deviceArray[i]].GPP &&
                (allDevices[deviceArray[i]].DSP != undefined)) {
            /* GPP + DSP */
            if (programs.match(/APP_CLIENT/)) {
            /* client implies server - we only support 'clients' on the GPP */
                addGPPOS(allDevices[deviceArray[i]].platform);
            }
            if (programs.match(/DSP_SERVER/)) {
                //Need to determine the ISA of the server 
                if (allDevices[deviceArray[i]].DSP == 'C64P') {
                    //C64P - based servers 
                    C64P = xdc.useModule('ti.targets.C64P');
                    addUniquePlatform(C64P, allDevices[deviceArray[i]].platform);
                }
                else if (allDevices[deviceArray[i]].DSP == 'C674') {
                    //C674 - based servers 
                    C674 = xdc.useModule('ti.targets.C674');
                    addUniquePlatform(C674, allDevices[deviceArray[i]].platform);
                }
                else {
                    throw('Unsupported DSP Server type (' +
                        allDevices[deviceArray[i]].DSP + ').  Only C64P and ' +
                        'C674 are currently supported.');
                }
            }
            if (programs.match(/APP_LOCAL/)) {
                /* Local apps build for both GPP and DSP */
                addGPPOS(allDevices[deviceArray[i]].platform);
                /* Need to determine the ISA of the server */
                if (allDevices[deviceArray[i]].DSP == 'C64P') {
                    /* C64P - based servers */
                    C64P = xdc.useModule('ti.targets.C64P');
                    addUniquePlatform(C64P, allDevices[deviceArray[i]].platform, "test_dsp");
                    if (elf == 1) {
                        C64Pelf = xdc.useModule('ti.targets.elf.C64P');
                        addUniquePlatform(C64Pelf, allDevices[deviceArray[i]].platform, "test_dsp");
                    }
                }
                else if (allDevices[deviceArray[i]].DSP == 'C674') {
                    /* C674 - based servers */
                    C674 = xdc.useModule('ti.targets.C674');
                    addUniquePlatform(C674, allDevices[deviceArray[i]].platform, "test_dsp");
                    if (elf == 1) {
                        C674elf = xdc.useModule('ti.targets.elf.C674');
                        addUniquePlatform(C674elf, allDevices[deviceArray[i]].platform, "test_dsp");
                    }
                }
                else {
                    throw('Unsupported DSP Server type (' +
                        allDevices[deviceArray[i]].DSP + ').  Only C64P and ' +
                        'C674 are currently supported.');
                }
            }
        } else if ((allDevices[deviceArray[i]].GPP) &&
                (allDevices[deviceArray[i]].DSP == undefined)) {
            /* GPP-only device */
            if (programs.match(/APP_LOCAL/)) {
                /* Local apps build for GPP */
                addGPPOS(allDevices[deviceArray[i]].platform);
            }
        } else if ((allDevices[deviceArray[i]].GPP == false) &&
                (allDevices[deviceArray[i]].DSP != undefined)) {
            /* DSP-only device */
            if (programs.match(/APP_LOCAL/)) {
                /* Local apps build for DSP */
                /* Need to determine the ISA of the server */
                if (allDevices[deviceArray[i]].DSP == 'C64P') {
                    /* C64P - based servers */
                    C64P = xdc.useModule('ti.targets.C64P');
                    addUniquePlatform(C64P, allDevices[deviceArray[i]].platform, "test_dsp");
                    if (elf == 1) {
                        C64Pelf = xdc.useModule('ti.targets.elf.C64P');
                        addUniquePlatform(C64Pelf, allDevices[deviceArray[i]].platform, "test_dsp");
                    }

                }
                else if (allDevices[deviceArray[i]].DSP == 'C674') {
                    /* C674 - based servers */
                    C674 = xdc.useModule('ti.targets.C674');
                    addUniquePlatform(C674, allDevices[deviceArray[i]].platform, "test_dsp");
                    if (elf == 1) {
                        C674elf = xdc.useModule('ti.targets.elf.C674');
                        addUniquePlatform(C674elf, allDevices[deviceArray[i]].platform, "test_dsp");
                    }

                }
                else {
                    throw('Unsupported DSP Server type (' +
                        allDevices[deviceArray[i]].DSP + ').  Only C64P and ' +
                        'C674 are currently supported.');
                }
            }
        } else {
            throw('Unsupported device configuration!  GPP+DSP, GPP-only, and' +
                    ' DSP-only are supported.');
        }
        if (allDevices[deviceArray[i]].ARM != undefined) {
            if (allDevices[deviceArray[i]].ARM == 'v7M') {
                M3 = xdc.useModule("ti.targets.arm.M3");
                addUniquePlatform(M3, allDevices[deviceArray[i]].platform, "test_dsp");
                if (elf == 1) {
                    M3elf = xdc.useModule("ti.targets.arm.elf.M3");
                    addUniquePlatform(M3elf, allDevices[deviceArray[i]].platform, "test_dsp");
                }
            }
            else {
            }
        }
            
    } else {
        throw('Unsupported DEVICE "' + deviceArray[i] + '".  Please check ' +
            'the DEVICES macro set in xdcpaths.mak.');
    }
}
/*
 *  @(#) ti.sdo.fc.examples.buildutils; 1, 0, 0,48; 3-1-2011 18:57:51; /db/atree/library/trees/fc/fc-n29x/src/ xlibrary

 */

